@@ -2,13 +2,13 @@ package MsOffice::Word::Surgeon::Text;
 use 5.24.0;
 use Moose;
 use MooseX::StrictConstructor;
+use MsOffice::Word::Surgeon::Carp;
 use MsOffice::Word::Surgeon::Utils qw(maybe_preserve_spaces is_at_run_level encode_entities);
-use Carp::Clan                     qw(^MsOffice::Word::Surgeon); # will import carp, croak, etc.
 
 use namespace::clean -except => 'meta';
 
 
-our $VERSION = '2.05';
+our $VERSION = '2.06';
 
 #======================================================================
 # ATTRIBUTES
